/**
 * config.development.js
 * This file contains the initialization of the configuration.
 */
'use strict';
const path = require('path');
const { logger } = require('./utils/logger.js');

var config = {
  app: {
    port: process.env.PORT || 8080
  },
  hooks: {
    debugEnabled: process.env.HooksDebugEnabled ? (process.env.HooksDebugEnabled === 'Y') : true,
    operations: [
      'createRequest',
      'cancelRequest',
      'deleteRequest',
      'updateRequestDataset',
      'completeAction',
      'cancelAction',
      'cancelActionAssignment',
      'assignAction',
      'createFavorite',
      'updateFavorite',
      'deleteFavorite',
      'createDelegation',
      'updateDelegation',
      'deleteDelegation',
      'createUser',
      'updateUser',
      'deleteUser',
      'createGroup',
      'updateGroup',
      'deleteGroup',
      'addGroupsToUser',
      'removeGroupsFromUser',
      'addUsersToGroup',
      'removeUsersFromGroup',
      'addComment',
      'removeComment',
      'cancelRequestActionsByName',
      'updateViewerPreferences',
      'getUser',
      'getUsers',
      'getActivities',
      'getActivity',
      'getApplication',
      'getApplications',
      'getApplicationParameter',
      'getApplicationParameters',
      'getFavorite',
      'getFavorites',
      'getActions',
      'getAction',
      'canAssignAction',
      'canAssignUserToAction',
      'canCancelActionAssignment',
      'getActionAssignableUsers',
      'getParticipant',
      'getGlobalParticipant',
      'getLocalProcessParticipant',
      'getParticipants',
      'getProcessParticipants',
      'getLocalProcessParticipantUserAssociations',
      'getLocalProcessParticipanCoordinatorAssociations',
      'getLocalProcessParticipanGroupAssociations',
      'getLocalProcessParticipanDirectoryAssociations',
      'getGlobalParticipantUserAssociations',
      'getGlobalParticipantCoordinatorAssociations',
      'getGlobalParticipanGroupAssociations',
      'getGlobalParticipanDirectoryAssociations',
      'getProcesses',
      'getProcess',
      'getLaunchableProcesses',
      'getProcessDataset',
      'getProcessData',
      'getRequests',
      'getRequest',
      'canCancelRequest',
      'canDeleteRequest',
      'getView',
      'getViews',
      'getProcessParticipantProfile',
      'getProcessParticipantProfiles',      
      'getDirectory',
      'getDirectories',
      'getGroup',
      'getGroups',
      'getComments',
      'getComment',
      'getCommentsByRequest',
      'getDelegations',
      'getDelegation',
      'getDelegators',
      'getDelegableProcessParticipants',
      'getDelegableProcessParticipant',
      'getRequestDataset',
      'getRequestData',
      'getActionData',
      'getActionDataset',
      'getCategory',
      'getCategories',
      'getServer'
    ]
  },
  workflowService: {
    debugEnabled: process.env.HooksDebugEnabled ? (process.env.HooksDebugEnabled === 'Y') : true,
    configFilePath: path.normalize(__dirname + '/../web.config'),
    serviceAssemblyFilePath: path.normalize(__dirname + '/../bin/Advantys.workflow.dll'),
    logHandler: (text) => logger('DEBUG', text),
    operationTimeout: process.env.HooksOperationTimeout ? parseInt(process.env.HooksOperationTimeout) : 30000,
    maxOperationCount: process.env.HooksMaxOperationCount ? parseInt(process.env.HooksMaxOperationCount) : 10000,
    maxInputFileSize: process.env.HooksMaxInputFileSize ? parseInt(process.env.HooksMaxInputFileSize) : 10240,
    inputFileDownloadTimeout: process.env.HooksInputFileDownloadTimeout ? parseInt(process.env.HooksInputFileDownloadTimeout) : 100000,
    maxOperationRetries: process.env.HooksMaxOperationRetries ? parseInt(process.env.HooksMaxOperationRetries) : 20,
    operationRetryDelay: process.env.HooksOperationRetryDelay ? parseInt(process.env.HooksOperationRetryDelay) : 200 
  },
  test: {
    databaseConnection: {
      server: process.env.TestDbServer ? process.env.TestDbServer : 'localhost',
      adminUser: process.env.TestDbAdminUserName ? process.env.TestDbAdminUserName : 'sa',
      adminPassword: process.env.TestDbAdminUserPassword ? process.env.TestDbAdminUserPassword : 'Admin2017$',
      user:process.env.TestDbUserName ? process.env.TestDbUserName : 'WFGEN_TEST_USER',
      password:process.env.TestDbPassword ? process.env.TestDbPassword : 'Admin2017$',
      database: process.env.TestDbName ? process.env.TestDbName : 'WFGEN_TEST',
      port: process.env.TestDbPort ? process.env.TestDbPort : 1433
    },
    testInstallScriptPath: process.env.TestInstallScriptPath ? process.env.TestInstallScriptPath : 'C:\\Temp',
    server: {
      port: null
    }
  }
};

module.exports = config;
