var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('AssignAction', function () {

  it('should assign an action', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'assignAction',
        'args': {
          'input': {
            'id': 'QWN0aW9uOjUtLS0x',
            'assigneeId': 'VXNlcjoy'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('requestNumber');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        res.body.requestNumber.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });

  it("should return the action(ID: 'QWN0aW9uOjUtLS0x') assigned to a user (ID: 'VXNlcjoy')", function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': 'QWN0aW9uOjUtLS0x'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateActionProperties(res.body);

        expect(res.body).to.include({ id: 'QWN0aW9uOjUtLS0x', status: 'OPEN', assigneeId: 'VXNlcjoy' });

        done();
      });
  });

  it('should assign an action in delegation mode', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'assignAction',
        'args': {
          'input': {
            'id': 'QWN0aW9uOjM3LS0tMQ==',
            'assigneeId': 'VXNlcjoz',
            'onBehalfOf': 'VXNlcjoy'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).not.to.be.a('null');
        res.body.onBehalfOf.should.be.equal('VXNlcjoy');
        res.body.onBehalfOfUserName.should.be.equal('mark_twain');

        done();
      });
  });

  it("should return the action(ID: 'QWN0aW9uOjM3LS0tMQ==') assigned to a user (ID: 'VXNlcjoz') in delegation mode", function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': 'QWN0aW9uOjM3LS0tMQ==',
          'onBehalfOf': 'VXNlcjoy'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateActionProperties(res.body);

        expect(res.body).to.include({ id: 'QWN0aW9uOjM3LS0tMQ==', status: 'OPEN', assigneeId: 'VXNlcjoz' });

        done();
      });
  });
});
