var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('UpdateDelegation', function () {
  it('should update the delegation with a different process participant', function (done) {
    chai.request(url)
    .post('/')
    .send({
      'operation': 'updateDelegation',
      'args': {
        'input': {
          'id' : 'RGVsZWdhdGlvbjo0',
          'processId': 'UHJvY2Vzczoy',
          'participantId': 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NQ=='
        }
      }
    })
    .end(function (err, res) {
      res.should.have.status(200);
      res.should.be.json;
      res.body.should.have.property('clientMutationId');
      res.body.should.have.property('delegationId');

      expect(res.body).to.include({ delegationId:'RGVsZWdhdGlvbjo0', clientMutationId:null });
      done();
    });
  });
  it('should verify the updated delegation\'s data', function (done) {
    chai.request(url)
    .post('/')
    .send({
      'operation': 'getDelegation',
      'args': {
          'id': 'RGVsZWdhdGlvbjo0'
      }
    })
    .end(function (err, res) {

      res.should.have.status(200);
      res.should.be.json;

      expect(res.body).to.include({ "id": "RGVsZWdhdGlvbjo0", "delegatorId": "VXNlcjoy", "delegateeId": "VXNlcjoz", "start": null, "end": null, "processId": "UHJvY2Vzczoy", "participantId": "TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NQ==" });

      done();
    });
  });

  it('should update a delegation\'s dates', function (done) {
    chai.request(url)
    .post('/')
    .send({
      'operation': 'updateDelegation',
      'args': {
        'input': {
          'id' : 'RGVsZWdhdGlvbjoz',
          'start': "2017-04-23T18:25:43.000Z",
          'end': "2018-11-23T18:25:43.000Z",
        }
      }
    })
    .end(function (err, res) {
      res.should.have.status(200);
      res.should.be.json;
      res.body.should.have.property('clientMutationId');
      res.body.should.have.property('delegationId');

      expect(res.body).to.include({ delegationId:'RGVsZWdhdGlvbjoz', clientMutationId:null });
      done();
    });
  });
  it('should verify the updated delegation\'s data', function (done) {
    chai.request(url)
    .post('/')
    .send({
      'operation': 'getDelegation',
      'args': {
          'id': 'RGVsZWdhdGlvbjoz'
      }
    })
    .end(function (err, res) {

      res.should.have.status(200);
      res.should.be.json;

      expect(res.body).to.include({ "id": "RGVsZWdhdGlvbjoz", "delegatorId": "VXNlcjoz", "delegateeId": "VXNlcjoy", "start": "2017-04-23T18:25:43.000Z", "end": "2018-11-23T18:25:43.000Z", "processId": null, "participantId": null });
      done();
    });
  });
});