var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

var favoriteId = null;

describe('CreateFavorite', function () {

  it('should create a favorite for a process using the default process description', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'createFavorite',
        'args': {
          'input': {
            'itemId': 'UHJvY2Vzczoy'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('favoriteId');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.favoriteId.should.be.a('string');

        favoriteId = res.body.favoriteId;

        done();
      });
  });

  it("should return the favorite created for a process using the default process description", function (done) {
    if (favoriteId === null) {
      this.skip();
    }
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getFavorite',
        'args': {
          'id': favoriteId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateFavoriteProperties(res.body);

        expect(res.body).to.include({ id: favoriteId, itemId: 'UHJvY2Vzczoy', description: '2 levels approval' });

        done();
      });
  });

  it('should create a favorite for a process using a new process description value', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'createFavorite',
        'args': {
          'input': {
            'itemId': 'UHJvY2Vzczoz',
            'description': 'New process description'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('favoriteId');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.favoriteId.should.be.a('string');

        favoriteId = res.body.favoriteId;

        done();
      });
  });

  it("should return the favorite created for a process using a new process description value", function (done) {
    if (favoriteId === null) {
      this.skip();
    }
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getFavorite',
        'args': {
          'id': favoriteId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateFavoriteProperties(res.body);

        expect(res.body).to.include({ id: favoriteId, itemId: 'UHJvY2Vzczoz', description: 'New process description' });

        done();
      });
  });

  it('should create a favorite for a view using the default view description', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'createFavorite',
        'args': {
          'input': {
            'itemId': 'VmlldzoxLS0tMQ=='
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('favoriteId');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.favoriteId.should.be.a('string');

        favoriteId = res.body.favoriteId;

        done();
      });
  });

  it("should return the favorite created for a process using the default view description", function (done) {
    if (favoriteId === null) {
      this.skip();
    }
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getFavorite',
        'args': {
          'id': favoriteId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateFavoriteProperties(res.body);

        expect(res.body).to.include({ id: favoriteId, itemId: 'VmlldzoxLS0tMQ==', description: 'Test_Requests' });

        done();
      });
  });

  it('should create a favorite for a view using a new view description value', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'createFavorite',
        'args': {
          'input': {
            'itemId': 'VmlldzoxLS0tMw==',
            'description': 'New view description'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('favoriteId');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.favoriteId.should.be.a('string');

        favoriteId = res.body.favoriteId;

        done();
      });
  });

  it("should return the favorite created for a process using a new view description value", function (done) {
    if (favoriteId === null) {
      this.skip();
    }
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getFavorite',
        'args': {
          'id': favoriteId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateFavoriteProperties(res.body);

        expect(res.body).to.include({ id: favoriteId, itemId: 'VmlldzoxLS0tMw==', description: 'New view description' });

        done();
      });
  });
});
