var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

var favoriteId = null;

describe('UpdateFavorite', function () {
  
  it('should update a favorite process description', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'updateFavorite',
        'args': {
          'input': {
            'id': 'RmF2b3JpdGU6MQ==',
            'description': 'Updated process description'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('favoriteId');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.favoriteId.should.be.a('string');

        favoriteId = res.body.favoriteId;

        done();
      });
  });

  it("should return the favorite with the process description updated", function (done) {
    if (favoriteId === null) {
      this.skip();
    }
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getFavorite',
        'args': {
          'id': favoriteId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateFavoriteProperties(res.body);

        expect(res.body).to.include({ id: favoriteId, itemId: 'UHJvY2Vzczox', description: 'Updated process description' });

        done();
      });
  });

  it('should update a favorite view description', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'updateFavorite',
        'args': {
          'input': {
            'id': 'RmF2b3JpdGU6NQ==',
            'description': 'Updated view description'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('favoriteId');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.favoriteId.should.be.a('string');

        favoriteId = res.body.favoriteId;

        done();
      });
  });

  it("should return the favorite with the view description updated", function (done) {
    if (favoriteId === null) {
      this.skip();
    }
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getFavorite',
        'args': {
          'id': favoriteId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateFavoriteProperties(res.body);

        expect(res.body).to.include({ id: favoriteId, itemId: 'VmlldzoxLS0tMQ==', description: 'Updated view description' });

        done();
      });
  });
});
