var chai = require('chai');
var chaiHttp = require('chai-http');
var things = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(things);

var groupId = null;

describe('RemoveUsersFromGroup', function () {

    it('should remove users from a group', function (done) {
      chai.request(url)
      .post('/')
      .send({
      'operation': 'removeUsersFromGroup',
        'args': {
          'input': {
            'id': 'R3JvdXA6MQ==',
            'userIds': ["VXNlcjoy"]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('groupId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.groupId).to.not.be.null;

        groupId = res.body.groupId;

        done();
      });
    });

    it('should return the users associated to a group', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'groupId': groupId
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 1);

            expect(res.body.items[0]).to.include({ id: "VXNlcjox", userName: 'wfgen_admin', lastName: "Administrator", directoryId: "RGlyZWN0b3J5OjE=" });

            done();
        });
    });

    it('should get an error when the user does not exist in the group\'s directory', function (done) {
        chai.request(url)
        .post('/')
        .send({
        'operation': 'removeUsersFromGroup',
          'args': {
            'input': {
              'id': 'R3JvdXA6MQ==',
              'userIds': ["VXNlcjo0"]
            }
          }
        })
        .end(function (err, res) {
            res.should.have.status(500);
            res.should.be.json;

            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The user ID \'4\' does not exist or is not in the same directory as the group.');

            done();
        });
    });
});
