var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

var groupId = null;

describe('UpdateGroup', function () {
  
    it('should update the group with a different name', function (done) {
      chai.request(url)
      .post('/')
      .send({
      'operation': 'updateGroup',
        'args': {
          'input': {
            'id': 'R3JvdXA6NQ==',
            'name': 'Group EE',
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('groupId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.groupId).to.not.be.null;

        groupId = res.body.groupId;

        done();
      });
    });

    it('should return the updated group with the new name ', function (done) {
      if (groupId === null) {
        this.skip();
      }
      chai.request(url)
      .post('/')
      .send({
        'operation': 'getGroup',
        'args': {
          'id': groupId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateGroupProperties(res.body);

        expect(res.body).to.include({ id: groupId, name: 'Group EE', directoryId: "RGlyZWN0b3J5OjI=" });
        
        done();
      });
    });

    it('should update the group with a new directory', function (done) {
      chai.request(url)
      .post('/')
      .send({
      'operation': 'updateGroup',
        'args': {
          'input': {
            'id': 'R3JvdXA6NA==',
            'directoryId': 'RGlyZWN0b3J5OjM='
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('groupId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.groupId).to.not.be.null;

        groupId = res.body.groupId;

        done();
      });
    });

    it('should return the updated group with the new directory ', function (done) {
        if (groupId === null) {
          this.skip();
        }
        chai.request(url)
        .post('/')
        .send({
          'operation': 'getGroup',
          'args': {
            'id': groupId
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;
  
          utils.validateGroupProperties(res.body);
  
          expect(res.body).to.include({ id: groupId, name: 'Group D', directoryId: "RGlyZWN0b3J5OjM=" });
          
          done();
        });
    });

    it('should get an error when the directory is updated and the same group name exists in the specified directory', function (done) {
        chai.request(url)
        .post('/')
        .send({
        'operation': 'updateGroup',
          'args': {
            'input': {
              'id': 'R3JvdXA6Ng==',
              'directoryId': 'RGlyZWN0b3J5OjE='
            }
          }
        })
        .end(function (err, res) {
            res.should.have.status(500);
            res.should.be.json;

            res.body.should.have.property('error');
            expect(res.body.error).to.contain('Another group with the same name exists in the specified directory');

            done();
        });
    });
});
