var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('DeleteRequest', function () {

  it('should delete a request using the request number', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'deleteRequest',
        'args': {
          'input': {
            'number': 2
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        expect(res.body.clientMutationId).to.be.a('null');

        done();
      });
  });

  it("should verify the deleted request", function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'number': 2
        }
      })
      .end(function (err, res) {
        res.should.have.status(500);
        res.should.be.json;
        res.body.should.have.property('error');
        res.body.error.should.be.a('string');
        expect(res.body.error).to.contain('The request does not exist.');

        done();
      });
  });

  it('should get an error when deleting a request that does not exist', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'deleteRequest',
        'args': {
          'input': {
            'number': 5000
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(500);
        res.should.be.json;
        res.body.should.have.property('error');
        res.body.error.should.be.a('string');
        expect(res.body.error).to.contain('The request ID or number does not exist.');

        done();
      });
  });

  it('should get an error when request number is not specified', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'deleteRequest',
        'args': {
          'input': {
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(400);
        res.should.be.json;
        res.body.should.have.property('error');
        res.body.error.should.be.a('string');
        expect(res.body.error).to.contain('A request ID or number is required.');

        done();
      });
  });
  
  it('should delete a request using the request ID', function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'deleteRequest',
        'args': {
          'input': {
            'id': 'UmVxdWVzdDoyMQ=='
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        expect(res.body.clientMutationId).to.be.a('null');

        done();
      });
  });

  it("should verify the deleted request", function (done) {
    chai.request(url)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'number': 21
        }
      })
      .end(function (err, res) {
        res.should.have.status(500);
        res.should.be.json;
        res.body.should.have.property('error');
        res.body.error.should.be.a('string');
        expect(res.body.error).to.contain('The request does not exist.');

        done();
      });
  });
});
