var chai = require('chai');
var chaiHttp = require('chai-http');
var things = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(things);

var userId = null;

describe('AddGroupsToUser', function () {

    it('should add groups to a user', function (done) {
      chai.request(url)
      .post('/')
      .send({
      'operation': 'addGroupsToUser',
        'args': {
          'input': {
            'id': 'VXNlcjo2',
            'groupIds': ["R3JvdXA6Mw==", "R3JvdXA6Nw=="]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.userId).to.not.be.null;

        userId = res.body.userId;

        done();
      });
    });

    it('should return the groups associated to a user', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getGroups',
            'args': {
                'userId': userId,
                'userAsMember': true
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateGroupPageListProperties(res.body, 2);

            expect(res.body.items[0]).to.include({ id: "R3JvdXA6Mw==", name: 'Group C', description: "Group C", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[1]).to.include({ id: "R3JvdXA6Nw==", name: 'Group F', description: "Group F", directoryId: "RGlyZWN0b3J5OjE=" });

            done();
        });
    });

    it('should get an error when the group does not exist in the user\'s directory', function (done) {
        chai.request(url)
        .post('/')
        .send({
        'operation': 'addGroupsToUser',
          'args': {
            'input': {
              'id': 'VXNlcjoz',
              'groupIds': ["R3JvdXA6NA=="]
            }
          }
        })
        .end(function (err, res) {
            res.should.have.status(500);
            res.should.be.json;

            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The group ID \'4\' does not exist or is not in the same directory as the user.');

            done();
        });
    });
});
