var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetActionDataset', function () {

    it('should get the dataset of an action given its ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActionDataset',
            'args' : {
                'actionId' : 'QWN0aW9uOjYtLS0x'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionDataPageListProperties(res.body, 5);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uRGF0YTo2LS0tMS0tLTM=', name: 'FORM_ARCHIVE'});
            expect(res.body.items[4]).to.include({ id:'QWN0aW9uRGF0YTo2LS0tMS0tLTU=', name: 'REQUEST_SUBJECT'});

            done();
        });
    });

    it('should get the dataset of an action given its ID filtered by an array of names', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActionDataset',
            'args' : {
                'actionId' : 'QWN0aW9uOjE1LS0tMQ==',
                'filter' : {
                    'names' : ["FORM_DATA", "REQUEST_ATTACHMENT"]
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionDataPageListProperties(res.body, 2);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uRGF0YToxNS0tLTEtLS0y', name: 'FORM_DATA'});
            expect(res.body.items[1]).to.include({ id:'QWN0aW9uRGF0YToxNS0tLTEtLS03', name: 'REQUEST_ATTACHMENT'});

            done();
        });
    });

    it('should get the dataset of an action given its ID filtered by type', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActionDataset',
            'args' : {
                'actionId' : 'QWN0aW9uOjE0LS0tMQ==',
                'filter' : {
                    'type' : 'TEXT'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionDataPageListProperties(res.body, 2);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uRGF0YToxNC0tLTEtLS00', name: 'FORM_DRAFT'});
            expect(res.body.items[1]).to.include({ id:'QWN0aW9uRGF0YToxNC0tLTEtLS01', name: 'REQUEST_SUBJECT'});

            done();
        });
    });
});
