var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetActions', function () {

    it('should get actions with as:ASSIGNEE filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'ASSIGNEE'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 20);
            
            // Values validation
            res.body.items.should.not.all.have.property('assigneeId', null);

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES'});
            expect(res.body.items[19]).to.include({ id:'QWN0aW9uOjI1LS0tMQ==', name: '1-INITIATES'});

            done();
        });
    });

    it('should get actions with as:ASSIGNOR filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'ASSIGNOR'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 5);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[4]).to.include({ id:'QWN0aW9uOjUtLS0x', name: '1-INITIATES'});

            done();
        });
    });

    it('should get actions with isSelfService:true filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isSelfService': true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 0);            

            // TODO
            // Values validation

            done();
        });
    });

    it('should get actions with isSelfService:true and as:ASSIGNEE filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isSelfService': true,
                    'as': 'ASSIGNEE'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 0);

            // TODO
            // Values validation
            //res.body.items.should.not.all.have.property('assigneeId', null);

            done();
        });
    });

    it('should get actions with as:SUPERVISOR filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'SUPERVISOR'
                },
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 40);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[39]).to.include({ id:'QWN0aW9uOjQwLS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get actions without filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 40);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[39]).to.include({ id:'QWN0aW9uOjQwLS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get actions with userAsMember:true arg', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'userAsMember': true,
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 25);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[24]).to.include({ id:'QWN0aW9uOjI1LS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get actions by request ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestId': 'UmVxdWVzdDoyNg=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 1);

            // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDoyNg==');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjI2LS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get actions with status:CLOSED filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'status': 'CLOSED'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            //validateActionProperties(res, 33);

            // TODO
            // Values validation
            // res.body.items.should.all.have.property('status', 'CLOSED');
            
            done();
        });
    });

    it('should get actions with assigneeId:"mark_twain" filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'assigneeId': 'VXNlcjoy'
                },
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 10);

            // Values validation
            res.body.items.should.all.have.property('assigneeId', 'VXNlcjoy');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjI2LS0tMQ==', name: '1-INITIATES'});
            expect(res.body.items[9]).to.include({ id:'QWN0aW9uOjM1LS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get actions with processName:"TEST_SIMPLE_APPROVAL_LOOP" filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'processName': 'TEST_SIMPLE_APPROVAL_LOOP'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 19);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjE2LS0tMQ==', name: '1-INITIATES'});
            expect(res.body.items[18]).to.include({ id:'QWN0aW9uOjM1LS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get all my actions in progress that are past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'status' : 'OPEN',
                    'as': 'ASSIGNEE',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all actions as assignor that are or were past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'ASSIGNOR',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all my team actions that are or were past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isSelfService': true,
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all my team actions as assignee that are or were past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isSelfService': true,
                    'as': 'ASSIGNEE',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all actions which I supervise that are or were past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'SUPERVISOR',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all actions by request ID that are or were past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'requestId': 'UmVxdWVzdDo2',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all actions that are or were past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all actions that are or were not past due', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isDueDateExceeded' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateActionPageListProperties(res.body, 0);

            done();
        });
    });
    
    it('should get actions with as:ASSIGNOR and createdSince filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNOR',
                    'createdSince': '2017-06-01T03:13:44.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 2);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:13:44.000Z'));
            
            done();
        });
    });

    it('should get actions with isSelfService:true and createdSince filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'isSelfService': true,
                    'createdSince': '2017-06-01T03:17:00.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO when adding data

            utils.validateActionPageListProperties(res.body, 0);

            // Values validation
            //res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:17:00.000Z'));
            
            done();
        });
    });

    it('should get actions with createdSince filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'createdSince': '2017-06-01T03:19:30.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 11);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:19:30.000Z'));
            
            done();
        });
    });
    
    it('should get an error when the filter createSince does not contain a valid DateTime', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'isSelfService': true,
                    'as': 'ASSIGNEE',
                    'createdSince': 'invalid date'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The createdSince filter does not contain a valid DateTime.');
  
            done();
        });
    });

    it('should get actions with as:ASSIGNEE and assignedSince filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNEE',
                    'assignedSince': '2017-06-01T03:13:44.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO when adding data

            utils.validateActionPageListProperties(res.body, 0);

            // Values validation
            //res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:13:44.000Z'));
            
            done();
        });
    });

    it('should get actions with as:ASSIGNOR and assignedSince filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNOR',
                    'assignedSince': '2017-06-01T03:13:40.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO when adding data

            utils.validateActionPageListProperties(res.body, 0);
            
            // Values validation
            //res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:13:44.000Z'));

            done();
        });
    });

    it('should get actions with as:SUPERVISOR and assignedSince filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'SUPERVISOR',
                    'assignedSince': '2017-06-01T03:13:40.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO when adding data

            utils.validateActionPageListProperties(res.body, 0);
            
            // Values validation
            //res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:13:44.000Z'));

            done();
        });
    });

    it('should get actions with assignedSince filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'assignedSince': '2017-06-01T03:13:40.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO when adding data

            utils.validateActionPageListProperties(res.body, 0);
            
            // Values validation
            //res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:13:44.000Z'));

            done();
        });
    });


    it('should get an error when the filter assignedSince does not contain a valid DateTime', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNOR',
                    'assignedSince': 'invalid date'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The assignedSince filter does not contain a valid DateTime.');
  
            done();
        });
    });

    it('should get actions with viewerAsMember filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
               'viewerAsMember': true,
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 40);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[39]).to.include({ id:'QWN0aW9uOjQwLS0tMQ==', name: '1-INITIATES'});

            done();
        });
    });
});
