var chai = require('chai');
var chaiHttp = require('chai-http');
var things = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(things);

describe('GetApplications', function () {

    it('should get all the applications', function (done) {
      chai.request(url)
        .post('/')
        .send({
          'operation': 'getApplications',
          'args': {
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;

          utils.validateApplicationPageListProperties(res.body, 16);

          done();
        });
    });
    it('should get applications filtered by their type', function (done) {
      chai.request(url)
        .post('/')
        .send({
          'operation': 'getApplications',
          'args': {
            'filter': {
              'type': 'ASSEMBLY'
            }
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;

          utils.validateApplicationPageListProperties(res.body, 14);

          res.body.items.should.all.have.property('url', null);
          res.body.items.should.all.have.property('wsdl', null);
          res.body.items.should.all.have.property('username', null);
          res.body.items.should.all.have.property('password', null);
          res.body.items.should.all.have.property('contentType', null);
          res.body.items.should.all.have.property('contextFormat', null);
          res.body.items.should.all.have.property('isSchemaDefinitionEmbedded', false);

          done();
        });
    });
    it('should get applications filtered by a part of their name', function (done) {
      chai.request(url)
        .post('/')
        .send({
          'operation': 'getApplications',
          'args': {
            'filter': {
              'nameContains': 'Send'
            }
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;

          utils.validateApplicationPageListProperties(res.body, 2);

          expect(res.body.items).to.have.deep.property('[0].name', 'SENDHTTPPOST');
          expect(res.body.items).to.have.deep.property('[1].name', 'SENDMESSAGE');

          done();
        });
    });
    it('should get applications filtered by a part of their description', function (done) {
      chai.request(url)
        .post('/')
        .send({
          'operation': 'getApplications',
          'args': {
            'filter': {
              'descriptionContains': 'Send'
            }
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;

          utils.validateApplicationPageListProperties(res.body, 2);

          expect(res.body.items).to.have.deep.property('[0].description', 'Send an HTTP POST request');
          expect(res.body.items).to.have.deep.property('[1].description', 'Send phone messages');

          done();
        });
    });
    it('should get applications filtered by the isActive property', function (done) {
      chai.request(url)
        .post('/')
        .send({
          'operation': 'getApplications',
          'args': {
            'filter': {
              'isActive': true
            }
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;

          utils.validateApplicationPageListProperties(res.body, 16);

          res.body.items.should.all.have.property('isActive', true);

          done();
        });
    });
    it('should get the parameters for a given application ID', function (done) {
      chai.request(url)
        .post('/')
        .send({
          'operation': 'getApplicationParameters',
          'args': {
            'id': 'QXBwbGljYXRpb246MjA='
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;
          res.body.should.have.property('totalCount');
          expect(res.body.totalCount).to.be.equal(6);
          res.body.should.have.property('items');
          expect(res.body.items).to.be.an('array');
          expect(res.body.items).to.have.lengthOf(6);
          res.body.items.should.all.have.property('id');
          res.body.items.should.all.have.property('name');
          expect(res.body.items).to.have.deep.property('[4].name', 'APP_URL');
          res.body.items.should.all.have.property('description');
          res.body.items.should.all.have.property('dataType');
          res.body.items.should.all.have.property('direction');
          res.body.items.should.all.have.property('isRequired');
          expect(res.body.items).to.have.deep.property('[4].isRequired', true);
          res.body.items.should.all.have.property('isDefault');
          done();
        });
    });
  });
  