var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetComments', function () {
    it('should get viewer\'s comments', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getComments',
            'args': {
                'userId': 'VXNlcjox' 
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateCommentPageListProperties(res.body, 3);

            expect(res.body.items[0]).to.include({ id:'Q29tbWVudDoxNi0tLTE=', message: 'test 9' });
            expect(res.body.items[2]).to.include({ id:'Q29tbWVudDoxLS0tMg==', message: 'test 2' });

            done();
        });
    });  
      
    it('should get a user\'s comments', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getComments',
            'args': {
                'userId': 'VXNlcjoy'
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateCommentPageListProperties(res.body, 2);

            expect(res.body.items[0]).to.include({ id:'Q29tbWVudDozNi0tLTE=', message: 'test user' });
            expect(res.body.items[1]).to.include({ id:'Q29tbWVudDoyNi0tLTE=', message: 'test user' });

            done();
        });
    });

    it('should get a user\'s comments INTERSECT viewer`s comments as member', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getComments',
            'args': {
                'userId': 'VXNlcjoy',
                'viewerAsMember' : true
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateCommentPageListProperties(res.body, 0);
            
            done();
        });
    });
});