var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);
    
describe('GetDelegableProcessParticipants', function () {
    it('should get viewer\'s delegable process participants', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 5);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS00' });
            expect(res.body.items[4]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjMtLS04' });

            done();
        });
    });

    it('should get viewer\'s delegable process participants filtered by process ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'processId' : 'UHJvY2Vzczoy'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 3);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS00' });
            expect(res.body.items[2]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS02' });

            done();
        });
    });

    it('should get viewer\'s delegable process participants filtered by participant ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'participantId' : 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NQ=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 1);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS01' });

            done();
        });
    });

    it('should get mark_twain\'s delegable process participants', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // Values validation
            expect(res.body.items).to.have.lengthOf(4);
            expect(res.body.totalCount).to.equal(4);

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 4);

            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS00' });
            expect(res.body.items[3]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjMtLS03' });

            done();
        });
    });
});
