var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);
    
describe('GetDelegations', function () {
    it('should get viewer\'s delegations with isActive:false filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getDelegations',
            'args': {
                'userId': 'VXNlcjox',
                'filter': {
                    'isActive' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegationPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get delegations with no filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getDelegations'
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegationPageListProperties(res.body, 4);

            // Values validation
            expect(res.body.items).to.deep.include({ id: 'RGVsZWdhdGlvbjox',
                delegatorId: 'VXNlcjoy',
                delegateeId: 'VXNlcjox',
                start: null,
                end: null,
                processId: null,
                participantId: null});
            expect(res.body.items).to.deep.include({ id: 'RGVsZWdhdGlvbjo0',
                delegatorId: 'VXNlcjoy',
                delegateeId: 'VXNlcjoz',
                start: null,
                end: null,
                processId: null,
                participantId: null});
        done();
        });
    });

    it('should get a user\'s delegations', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getDelegations',
            'args': {
                'userId' : 'VXNlcjoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegationPageListProperties(res.body, 2);

            // Values validation
             expect(res.body.items).to.deep.include({ id: 'RGVsZWdhdGlvbjox',
                delegatorId: 'VXNlcjoy',
                delegateeId: 'VXNlcjox',
                start: null,
                end: null,
                processId: null,
                participantId: null});
            expect(res.body.items).to.deep.include({ id: 'RGVsZWdhdGlvbjo0',
                delegatorId: 'VXNlcjoy',
                delegateeId: 'VXNlcjoz',
                start: null,
                end: null,
                processId: null,
                participantId: null});
        done();
        });
    });
});
