var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetParticipants', function () {
    
    it('should get participants without filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getParticipants'
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateParticipantPageListProperties(res.body, 8);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'R2xvYmFsUGFydGljaXBhbnQ6MQ==', name: 'DEFAULT_PROCESSMANAGER'});
            expect(res.body.items[7]).to.include({ id:'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6Nw==', name: 'REQUESTER'});

            done();
        });
    });

    it('should get participants with userAsMember:true arg', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getParticipants',
            'args': {
                'userAsMember': true
            }            
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateParticipantPageListProperties(res.body, 8);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'R2xvYmFsUGFydGljaXBhbnQ6MQ==', name: 'DEFAULT_PROCESSMANAGER'});
            expect(res.body.items[7]).to.include({ id:'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6Nw==', name: 'REQUESTER'});

            done();
        });
    });

    it('should get participants with isGlobal:true filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getParticipants',
            'args': {
                'filter': {
                    'isGlobal': true
                }
            }          
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateParticipantPageListProperties(res.body, 1);

            // Values validation
            res.body.items.should.all.have.property('processId', null);

            expect(res.body.items[0]).to.include({ id:'R2xvYmFsUGFydGljaXBhbnQ6MQ==', name: 'DEFAULT_PROCESSMANAGER'});

            done();
        });
    });

    it('should get participants with isGlobal:false filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getParticipants',
            'args': {
                'filter': {
                    'isGlobal': false
                }
            }            
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            utils.validateParticipantPageListProperties(res.body, 7);

            // Values validation
            res.body.items.should.not.all.have.property('processId', null);            

            expect(res.body.items[0]).to.include({ id:'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NQ==', name: 'LEVEL_1'});
            expect(res.body.items[6]).to.include({ id:'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6Nw==', name: 'REQUESTER'});

            done();
        });
    });
});
