var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetLaunchableProcesses', function () {
    
    it('should get viewer\'s launchable processes', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getLaunchableProcesses',
            'args' : {
                'userId' : 'VXNlcjox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            validateProcessProperties(res, 3);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczoy', name: 'TEST_2_LEVELS_APPROVAL'});
            expect(res.body.items[1]).to.include({ id:'UHJvY2Vzczox', name: 'TEST_SIMPLE_APPROVAL'});
            expect(res.body.items[2]).to.include({ id:'UHJvY2Vzczoz', name: 'TEST_SIMPLE_APPROVAL_LOOP'});

            done();
        });
    });

    it('should get mark_twain\'s launchable processes', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getLaunchableProcesses',
            'args' : {
                'userId' : 'VXNlcjoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            validateProcessProperties(res, 2);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczoy', name: 'TEST_2_LEVELS_APPROVAL'});
            expect(res.body.items[1]).to.include({ id:'UHJvY2Vzczoz', name: 'TEST_SIMPLE_APPROVAL_LOOP'});

            done();
        });
    });
});

function validateProcessProperties(res, totalCount)
{
    // Properties
    res.body.should.have.property('totalCount');
    res.body.should.have.property('items');

    // Count
    expect(res.body.items).to.have.lengthOf(totalCount);
    expect(res.body.totalCount).to.equal(totalCount);

    // Array validation
    res.body.items.should.be.a('array');

    res.body.items.should.all.have.property('id');
    res.body.items.should.all.have.property('name');
}
