var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('GetProcessDataset', function () {

    it('should get a process dataset given its ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getProcessDataset',
            'args': {
                'processId': 'UHJvY2Vzczoz'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            validateProcessDatasetProperties(res, 7);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzc0RhdGE6My0tLTI1', name: 'APPROVAL_DECISION'});
            expect(res.body.items[6]).to.include({ id:'UHJvY2Vzc0RhdGE6My0tLTI0', name: 'REQUEST_SUBJECT'});

            done();
        });
    });
});

function validateProcessDatasetProperties(res, totalCount)
{
    // Properties
    res.body.should.have.property('totalCount');
    res.body.should.have.property('items');

    // Count
    expect(res.body.items).to.have.lengthOf(totalCount);
    expect(res.body.totalCount).to.equal(totalCount);
    
    // Array validation
    res.body.items.should.be.a('array');

    res.body.items.should.all.have.property('id');
    res.body.items.should.all.have.property('name');
}