var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetProcesses', function () {
    
    it('should get all processes without filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getProcesses'
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 3);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczoy', name: 'TEST_2_LEVELS_APPROVAL'});
            expect(res.body.items[1]).to.include({ id:'UHJvY2Vzczox', name: 'TEST_SIMPLE_APPROVAL'});
            expect(res.body.items[2]).to.include({ id:'UHJvY2Vzczoz', name: 'TEST_SIMPLE_APPROVAL_LOOP'});

            done();
        });
    });

    it('should get processes with state:TEST filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getProcesses',
            'args': {
                'filter':{
                    'state': 'TEST'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 1);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczox', name: 'TEST_SIMPLE_APPROVAL'});

            done();
        });
    });
});
