var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
const config = require('../../../config.test');
var should = chai.should();
chai.use(chaiHttp);

describe('CanDeleteRequest', function(){

    it('should return a boolean representing if the viewer can delete a request(in test) given its ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'canDeleteRequest',
            'args' : {
                'id' : 'UmVxdWVzdDoyMw=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            expect(res.body).to.equal(true);
            done();
        });
    });

    it('should return a boolean representing if the viewer can delete a request(active) given its number', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'canDeleteRequest',
            'args' : {
                'number' : 1
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            expect(res.body).to.equal(true);
            done();
        });
    });

    it('should return a boolean representing if the user(being a process manager) can delete a request(test) given its ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'canDeleteRequest',
            'args' : {
                'userId' : 'VXNlcjo3',
                'id' : 'UmVxdWVzdDo2'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            expect(res.body).to.equal(true);
            done();
        });
    });

    it('should return a boolean representing if the user(being a process manager) can delete a request(active) given its number', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'canDeleteRequest',
            'args' : {
                'userId' : 'VXNlcjo3',
                'number' : 1
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // EngineAllowManagerToDeleteRealProcessInstances should be Y
            expect(res.body).to.equal(true);
            done();
        });
    });

    it('should return a boolean representing if the user(standard user) can delete a request in delegation mode', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'canDeleteRequest',
            'args' : {
                'id' : 'UmVxdWVzdDoy',
                'userId' : 'VXNlcjoz',
                'onBehalfOf' : 'VXNlcjox'
            }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;
  
          expect(res.body).to.equal(false);
          done();
        });
    });

    it('should get an error when the request ID does not exist', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'canDeleteRequest',
            'args' : {
                'id' : 'UmVxdWVzdDo5OTk5'
            }
        })
        .end(function (err, res) {
          res.should.have.status(500);
          res.should.be.json;
  
          res.body.should.have.property('error');
          expect(res.body.error).to.contain('The request ID or number does not exist.');

          done();
        });
    });
});
