var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetRequestDataset', function () {

    it('should get the dataset of a request given its ID', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequestDataset',
            'args' : {
                'requestId' : 'UmVxdWVzdDoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestDataPageListProperties(res.body, 8);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdERhdGE6Mi0tLTE0', name: 'FORM_ARCHIVE'});
            expect(res.body.items[7]).to.include({ id:'UmVxdWVzdERhdGE6Mi0tLTE2', name: 'REQUEST_SUBJECT'});

            done();
        });
    });

    it('should get the dataset of a request given its ID filtered by an array of names', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequestDataset',
            'args' : {
                'requestId' : 'UmVxdWVzdDo5',
                'filter' : {
                    'names' : ["FORM_ARCHIVE", "FORM_HTML"]
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestDataPageListProperties(res.body, 2);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdERhdGE6OS0tLTM=', name: 'FORM_ARCHIVE'});
            expect(res.body.items[1]).to.include({ id:'UmVxdWVzdERhdGE6OS0tLTE=', name: 'FORM_HTML'});

            done();
        });
    });

    it('should get the dataset of a request given its ID filtered by type', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequestDataset',
            'args' : {
                'requestId' : 'UmVxdWVzdDoz',
                'filter' : {
                    'type' : 'TEXT'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestDataPageListProperties(res.body, 4);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdERhdGE6My0tLTE1', name: 'FORM_DRAFT'});
            expect(res.body.items[3]).to.include({ id:'UmVxdWVzdERhdGE6My0tLTE2', name: 'REQUEST_SUBJECT'});

            done();
        });
    });
});
