var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetRequests', function () {

    it('should get all requests without filter and with page size', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'page' : {
                    'size' : 50,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 40);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDox', name: 'TEST_2_LEVELS_APPROVAL #1'});
            expect(res.body.items[39]).to.include({ id:'UmVxdWVzdDo0MA==', name: 'TEST_2_LEVELS_APPROVAL #40'});

            done();
        });
    });

    it('should get all requests with isTest:true filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'isTest' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 11);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo2', name: 'TEST_SIMPLE_APPROVAL #6'});
            expect(res.body.items[10]).to.include({ id:'UmVxdWVzdDoyMw==', name: 'TEST_SIMPLE_APPROVAL #23'});

            done();
        });
    });

    it('should get all requests with as:REQUESTER filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'as' : 'REQUESTER'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 25);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDox', name: 'TEST_2_LEVELS_APPROVAL #1'});
            expect(res.body.items[24]).to.include({ id:'UmVxdWVzdDoyNQ==', name: 'TEST_SIMPLE_APPROVAL_LOOP #25'});

            done();
        });
    });

    it('should get all requests with userAsMember arg', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'userAsMember' : true
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 25);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDox', name: 'TEST_2_LEVELS_APPROVAL #1'});
            expect(res.body.items[24]).to.include({ id:'UmVxdWVzdDoyNQ==', name: 'TEST_SIMPLE_APPROVAL_LOOP #25'});

            done();
        });
    });

    it('should get all requests by requesterID filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'requesterId' : 'VXNlcjoy'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 15);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDoyNg==', name: 'TEST_SIMPLE_APPROVAL_LOOP #26'});
            expect(res.body.items[14]).to.include({ id:'UmVxdWVzdDo0MA==', name: 'TEST_2_LEVELS_APPROVAL #40'});

            done();
        });
    });

    it('should get all requests by processName filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'processName' : 'TEST_SIMPLE_APPROVAL_LOOP'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 19);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDoxNg==', name: 'TEST_SIMPLE_APPROVAL_LOOP #16'});
            expect(res.body.items[18]).to.include({ id:'UmVxdWVzdDozNQ==', name: 'TEST_SIMPLE_APPROVAL_LOOP #35'});

            done();
        });
    });

    it('should get all requests from user "mark_twain"', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'userId' : 'VXNlcjoy',
                'userAsMember' : true
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 15);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDoyNg==', name: 'TEST_SIMPLE_APPROVAL_LOOP #26'});
            expect(res.body.items[14]).to.include({ id:'UmVxdWVzdDo0MA==', name: 'TEST_2_LEVELS_APPROVAL #40'});

            done();
        });
    });

    it('should get all requests from user "mark_twain" and viewerAsMember', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'userId' : 'VXNlcjoy',
                'userAsMember' : true,
                'viewerAsMember' : true
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all requests that are or were past due.', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateRequestPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all requests in progress that are past due.', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'status' : 'OPEN',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateRequestPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all my requests that are not past due.', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'as' : 'REQUESTER',
                    'isDueDateExceeded' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateRequestPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get all requests in progress which I supervise that are not past due.', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'as' : 'SUPERVISOR',
                    'status' : 'OPEN',
                    'isDueDateExceeded' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // TODO: Generate test data
            utils.validateRequestPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get requests with as:REQUESTER and createdSince filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'as': 'REQUESTER',
                    'createdSince': '2017-06-01T03:17:00.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 4);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:17:00.000Z'));
            
            done();
        });
    });

    it('should get requests with as:SUPERVISOR and createdSince filters', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'as': 'SUPERVISOR',
                    'createdSince': '2017-06-01T03:20:00.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 7);
            
            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:20:00.000Z'));
            
            done();
        });
    });

    it('should get requests with createdSince filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'createdSince': '2017-06-01T03:17:00.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 19);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:17:00.000Z'));
            
            done();
        });
    });
    
    it('should get an error when the filter createSince does not contain a valid DateTime', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'createdSince': 'invalid date'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The createdSince filter does not contain a valid DateTime.');
  
            done();
        });
    });
});
