var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetViews', function () {
    
    it('should get views without filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getViews',
            'args': {
                'userId': 'VXNlcjox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateViewPageListProperties(res.body, 3);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'VmlldzoxLS0tMg==', description: 'Requests'});
            expect(res.body.items[1]).to.include({ id:'VmlldzoxLS0tMw==', description: 'Supervisor_Requests'});
            expect(res.body.items[2]).to.include({ id:'VmlldzoxLS0tMQ==', description: 'Test_Requests'});

            done();
        });
    });

    it('should get views where userId != viewerId', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getViews',
            'args': {
                'userId': 'VXNlcjoz' 
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateViewPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get views with isShared:true filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getViews',
            'args': {
                'userId': 'VXNlcjox',
                'filter': {
                    'isShared': true
                }
            }              
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateViewPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get views with asAuthor:true filter', function (done) {
        chai.request(url)
        .post('/')
        .send({
            'operation': 'getViews',
            'args': {
                'userId': 'VXNlcjox',
                'filter': {
                    'asAuthor': true
                }
            }              
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateViewPageListProperties(res.body, 3);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'VmlldzoxLS0tMg==', description: 'Requests'});
            expect(res.body.items[1]).to.include({ id:'VmlldzoxLS0tMw==', description: 'Supervisor_Requests'});
            expect(res.body.items[2]).to.include({ id:'VmlldzoxLS0tMQ==', description: 'Test_Requests'});

            done();
        });
    });    
});
