var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var payload = require('./payload');
var should = chai.should();
chai.use(chaiHttp);

describe('Test the server.js file using application/x-www-form-urlencoded content type payloads', function () {
  it('should successfully send a http request using the POST method', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('GenericTest'))))
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.include({ id: 'UmVxdWVzdDo5', name: 'TEST_SIMPLE_APPROVAL #9' });
        done();
      });
  });
  it('should get an error when sending a http request using the PUT method', function (done) {
    chai.request(url)
      .put('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('GenericTest'))))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot PUT');
        done();
      });
  });
  it('should get an error when sending a http request using the GET method', function (done) {
    chai.request(url)
      .get('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('GenericTest'))))

      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot GET');
        done();
      });
  });
  it('should get an error when sending a http request using the DELETE method', function (done) {
    chai.request(url)
      .delete('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('GenericTest'))))

      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot DELETE');
        done();
      });
  });
  it('should get an error when the token is missing', function (done) {
    chai.request('http://localhost:' + port + '/wfgen/hooks/')
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('GenericTest'))))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot POST');
        done();
      });
  });
  it('should get an error when the JSON payload is invalid', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(payload.getPayload('InvalidJson')))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(400);
        res.should.be.json;
        expect(res.body.error).to.equal('Invalid JSON in payload.');
        done();
      });
  });
  it('should get an error when the payload key is not defined', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(encodeURIComponent(payload.getPayload('GenericPayload')))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(400);
        res.should.be.json;
        expect(res.body.error).to.equal('The payload key is not defined.');
        done();
      });
  });
  it('should get an error when there is no operation declared', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('MissingOperation'))))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(400);
        res.should.be.json;
        expect(res.body.error).to.equal('An operation is required.');
        done();
      });
  });
  it('should get an error when the requested operation name is invalid', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('InvalidOperation'))))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(400);
        res.should.be.json;
        expect(res.body.error).to.equal('Invalid operation value.');
        done();
      });
  });
  it('should get an error when the requested operation is not found', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(JSON.stringify(payload.getPayload('OperationNotFound'))))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.json;
        expect(res.body.error).to.equal('The requested operation was not found.');
        done();
      });
  });
  it('should get an error when the payload value is null', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/x-www-form-urlencoded')
      .send(
        'payload=' + encodeURIComponent(null))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(400);
        res.should.be.json;
        expect(res.body.error).to.equal('Invalid JSON.');
        done();
      });
  });
});
