var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var payload = require('./payload');
var should = chai.should();
chai.use(chaiHttp);

describe('Test the server.js file using application/json content type payloads', function () {
  it('should successfully send a http request using the POST method', function (done) {
    chai.request(url)
      .post('/')
      .set('content-type', 'application/json')
      .send(payload.getPayload('GenericTest'))
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.include({ id: 'UmVxdWVzdDo5', name: 'TEST_SIMPLE_APPROVAL #9' });
        done();
      });
  });
  it('should get an error when sending a http request using the PUT method', function (done) {
    chai.request(url)
      .put('/')
      .send(payload.getPayload('GenericTest'))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot PUT');
        done();
      });
  });
  it('should get an error when sending a http request using the GET method', function (done) {
    chai.request(url)
      .get('/')
      .send(payload.getPayload('GenericTest'))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot GET');
        done();
      });
  });
  it('should get an error when sending a http request using the DELETE method', function (done) {
    chai.request(url)
      .delete('/')
      .send(payload.getPayload('GenericTest'))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot DELETE');
        done();
      });
  });
  it('should get an error when the token is missing', function (done) {
    chai.request('http://localhost:' + port + '/wfgen/hooks/')
      .post('/')
      .send(payload.getPayload('GenericTest'))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.html;
        expect(res.text).to.contain('Cannot POST');
        done();
      });
  });
  it('should get an error when there is no operation declared', function (done) {
    chai.request(url)
      .post('/')
      .send(payload.getPayload('MissingOperation'))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(400);
        res.should.be.json;
        expect(res.body.error).to.equal('An operation is required.');
        done();
      });
  });
  it('should get an error when the requested operation name is invalid', function (done) {
    chai.request(url)
      .post('/')
      .send(payload.getPayload('InvalidOperation'))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(400);
        res.should.be.json;
        expect(res.body.error).to.equal('Invalid operation value.');
        done();
      });
  });
  it('should get an error when the requested operation is not found', function (done) {
    chai.request(url)
      .post('/')
      .send(payload.getPayload('OperationNotFound'))
      .end(function (err, res) {
        expect(res.statusCode).to.equal(404);
        res.should.be.json;
        expect(res.body.error).to.equal('The requested operation was not found.');
        done();
      });
  });
});
