const fs = require('fs');
const path = require('path');
const config = require('../../config.test');
const mkdirp = require('mkdirp');

// Copy database scripts
var copyScripts = function () {
  let destDir = path.join(__dirname + '/scripts', '');
  fs.access(destDir, (err) => {
    if (err) {
      fs.mkdirSync(destDir);
    }
    fs.readdir(config.test.testInstallScriptPath, (err, files) => {
      files.forEach(file => {
        if (file.indexOf('Create_WFG') > -1) {
          let src = path.join(config.test.testInstallScriptPath, file);
          copyFile(src, path.join(destDir, file));
        }
      });
    });
  });

  function copyFile (src, dest) {
    let readStream = fs.createReadStream(src, {encoding: 'binary'});

    readStream.once('error', (err) => {
      console.log(err);
    });
    readStream.once('end', () => {
      removeGoOnScript(dest);
    });
    readStream.pipe(fs.createWriteStream(dest));
  }
};

// Adapt script to T-SQL (by removing the GO statement that is not T-SQL but a SQL Server Management Studio artifact)
var removeGoOnScript = function (file) {
  return new Promise(function (fulfill, reject) {
    fs.readFile(file, 'utf8', function (err, data) {
      if (err) {
        reject(err);
        return console.log(err);
      }
      var result = data.replace(new RegExp('\\bGO\\b', 'gi'), '');
      fulfill(data);
      fs.writeFile(file, result, 'utf8', function (err) {
        if (err) return console.log(err);
      });
    });
  });
};

// Split triggers script into independent stored procedures
var splitTriggersScript = function (file) {
  return new Promise(function (resolve, reject) {
    let destDir = path.join('./scripts', '');

    fs.readFile(file, 'utf8', function (err, data) {
      if (err) {console.log(err); reject(err); }

      data = data.replace(/create trigger/ig, '{create trigger');
      data = data.split('{');

      resolve(data);
    });
  });
};

// Copy process definition webform
var copyProcessFiles = function () {
  var webformsPath = '../WfApps/WebForms/';
  fs.readdir(path.join(__dirname, '../webforms/'), (err, files) => {
    files.forEach(folder => {
      if (folder.indexOf('TEST_') > -1) {
        copyFolderRecursiveSync('./test/webforms/' + folder, webformsPath);
      }
    });
  });
  var designFilesPath = '../App_Data/Files/DataSet/design/2017/06/01';
  var filesToCopy = ['./test/files/1-1000.html', './test/files/12-1000.html', './test/files/20-1000.html'];
  if (!fs.existsSync(designFilesPath)) {
    mkdirp(designFilesPath, function () {
      filesToCopy.forEach(function (item) {
        copyFileSync(item, designFilesPath);
      });
    });
    return;
  }
  filesToCopy.forEach(function (item) {
    copyFileSync(item, designFilesPath);
  });
};

var copyFolderRecursiveSync = function (source, target) {
  var files = [];
  // check if folder needs to be created or integrated
  var targetFolder = path.join(target, path.basename(source));
  if (!fs.existsSync(targetFolder)) {
    fs.mkdirSync(targetFolder);
  }
  if (fs.lstatSync(source).isDirectory()) {
    files = fs.readdirSync(source);
    files.forEach(function (file) {
      var curSource = path.join(source, file);
      if (fs.lstatSync(curSource).isDirectory()) {
        copyFolderRecursiveSync(curSource, targetFolder);
      } else {
        copyFileSync(curSource, targetFolder);
      }
    });
  }
};

function copyFileSync (source, target) {
  var targetFile = target;
  if (fs.existsSync(target)) {
    if (fs.lstatSync(target).isDirectory()) {
      targetFile = path.join(target, path.basename(source));
    }
  }
  fs.writeFileSync(targetFile, fs.readFileSync(source));
}

// Delete the process definition webforms
var deleteProcessFiles = function () {
  var filesToDelete = ['TEST_2_LEVELS_APPROVAL', 'TEST_SIMPLE_APPROVAL', 'TEST_SIMPLE_APPROVAL_LOOP'];
  filesToDelete.forEach(function (item) {
    var dirPath = '../WfApps/WebForms/' + item;
    if (fs.existsSync(dirPath)) {
      removeDirectory(dirPath);
    }
  });
  var filesToDelete = ['1-1000.html', '12-1000.html', '20-1000.html'];
  filesToDelete.forEach(function (item) {
    var filePath = '../App_Data/Files/DataSet/design/2017/06/01/' + item;
    if (fs.existsSync(filePath)) {
      fs.unlinkSync(filePath);
    }
  });
};

function removeDirectory (dirPath) {
  try { var files = fs.readdirSync(dirPath);
  } catch(e) { return; }
  if (files.length > 0)
    for (var i = 0; i < files.length; i++) {
      var filePath = dirPath + '/' + files[i];
      if (fs.statSync(filePath).isFile())
        fs.unlinkSync(filePath);
      else
        removeDirectory(filePath);
  }
  fs.rmdirSync(dirPath);
}

module.exports = {
  copyScripts,
  splitTriggersScript,
  copyProcessFiles,
deleteProcessFiles};
